@@ -1,12 +1,8 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "claim.h"
-#include "../registry/registry_internals.h"
-#ifndef ACLK_NG
-#include "../aclk/legacy/aclk_common.h"
-#else
-#include "../aclk/aclk.h"
-#endif
+#include "registry/registry_internals.h"
+#include "aclk/aclk_api.h"
 
 char *claiming_pending_arguments = NULL;
 
@@ -138,8 +134,14 @@ void load_claiming_state(void)
     netdata_cloud_setting = 0;
 #else
     uuid_t uuid;
+
+    // Propagate into aclk and registry. Be kind of atomic...
+    appconfig_get(&cloud_config, CONFIG_SECTION_GLOBAL, "cloud base url", DEFAULT_CLOUD_BASE_URL);
+
     rrdhost_aclk_state_lock(localhost);
     if (localhost->aclk_state.claimed_id) {
+        if (aclk_connected)
+            localhost->aclk_state.prev_claimed_id = strdupz(localhost->aclk_state.claimed_id);
         freez(localhost->aclk_state.claimed_id);
         localhost->aclk_state.claimed_id = NULL;
     }
@@ -150,9 +152,6 @@ void load_claiming_state(void)
     }
     aclk_disable_runtime = 0;
 
-    // Propagate into aclk and registry. Be kind of atomic...
-    appconfig_get(&cloud_config, CONFIG_SECTION_GLOBAL, "cloud base url", DEFAULT_CLOUD_BASE_URL);
-
     char filename[FILENAME_MAX + 1];
     snprintfz(filename, FILENAME_MAX, "%s/cloud.d/claimed_id", netdata_configured_varlib_dir);
 
@@ -163,13 +162,23 @@ void load_claiming_state(void)
         freez(claimed_id);
         claimed_id = NULL;
     }
-    localhost->aclk_state.claimed_id = claimed_id;
+
+    if(claimed_id) {
+        localhost->aclk_state.claimed_id = mallocz(UUID_STR_LEN);
+        uuid_unparse_lower(uuid, localhost->aclk_state.claimed_id);
+    }
+
+    invalidate_node_instances(&localhost->host_uuid, claimed_id ? &uuid : NULL);
+    store_claim_id(&localhost->host_uuid, claimed_id ? &uuid : NULL);
+
     rrdhost_aclk_state_unlock(localhost);
     if (!claimed_id) {
         info("Unable to load '%s', setting state to AGENT_UNCLAIMED", filename);
         return;
     }
 
+    freez(claimed_id);
+
     info("File '%s' was found. Setting state to AGENT_CLAIMED.", filename);
     netdata_cloud_setting = appconfig_get_boolean(&cloud_config, CONFIG_SECTION_GLOBAL, "enabled", 1);
 #endif
