@@ -70,11 +70,9 @@ int do_macos_sysctl(int update_every, usec_t dt) {
         do_uptime               = config_get_boolean("plugin:macos:sysctl", "system uptime", 1);
     }
 
-    RRDSET *st;
+    RRDSET *st = NULL;
 
-    int system_pagesize = getpagesize(); // wouldn't it be better to get value directly from hw.pagesize?
-    int i, n;
-    int common_error = 0;
+    int i;
     size_t size;
 
     // NEEDED BY: do_loadavg
@@ -96,7 +94,6 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
     // NEEDED BY: do_tcp...
     struct tcpstat tcpstat;
-    uint64_t tcps_states[TCP_NSTATES];
 
     // NEEDED BY: do_udp...
     struct udpstat udpstat;
@@ -240,7 +237,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "System Load Average"
                             , "load"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 100
                             , (update_every < MIN_LOADAVG_UPDATE_EVERY) ? MIN_LOADAVG_UPDATE_EVERY : update_every
@@ -270,7 +267,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             do_swap = 0;
             error("DISABLED: system.swap");
         } else {
-            st = rrdset_find_localhost("system.swap");
+            st = rrdset_find_active_localhost("system.swap");
             if (unlikely(!st)) {
                 st = rrdset_create_localhost(
                         "system"
@@ -280,7 +277,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                         , NULL
                         , "System Swap"
                         , "MiB"
-                        , "macos"
+                        , "macos.plugin"
                         , "sysctl"
                         , 201
                         , update_every
@@ -332,7 +329,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                         iftot.ift_obytes += if2m->ifm_data.ifi_obytes;
                     }
                 }
-                st = rrdset_find_localhost("system.ipv4");
+                st = rrdset_find_active_localhost("system.ipv4");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "system"
@@ -342,7 +339,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 Bandwidth"
                             , "kilobits/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 500
                             , update_every
@@ -382,7 +379,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             error("DISABLED: ipv4.ecnpkts");
         } else {
             if (likely(do_tcp_packets)) {
-                st = rrdset_find_localhost("ipv4.tcppackets");
+                st = rrdset_find_active_localhost("ipv4.tcppackets");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -392,7 +389,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 TCP Packets"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 2600
                             , update_every
@@ -412,7 +409,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             // --------------------------------------------------------------------
 
             if (likely(do_tcp_errors)) {
-                st = rrdset_find_localhost("ipv4.tcperrors");
+                st = rrdset_find_active_localhost("ipv4.tcperrors");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -422,7 +419,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 TCP Errors"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 2700
                             , update_every
@@ -445,7 +442,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             // --------------------------------------------------------------------
 
             if (likely(do_tcp_handshake)) {
-                st = rrdset_find_localhost("ipv4.tcphandshake");
+                st = rrdset_find_active_localhost("ipv4.tcphandshake");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -455,7 +452,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 TCP Handshake Issues"
                             , "events/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 2900
                             , update_every
@@ -486,7 +483,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                                 tcpstat.tcps_persistdrop ||
                                                                 netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_tcpext_connaborts = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv4.tcpconnaborts");
+                st = rrdset_find_active_localhost("ipv4.tcpconnaborts");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -496,7 +493,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "TCP Connection Aborts"
                             , "connections/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 3010
                             , update_every
@@ -523,7 +520,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                         (tcpstat.tcps_rcvoopack ||
                                                          netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_tcpext_ofo = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv4.tcpofo");
+                st = rrdset_find_active_localhost("ipv4.tcpofo");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -533,7 +530,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "TCP Out-Of-Order Queue"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 3050
                             , update_every
@@ -557,7 +554,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                                 netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_tcpext_syscookies = CONFIG_BOOLEAN_YES;
 
-                st = rrdset_find_localhost("ipv4.tcpsyncookies");
+                st = rrdset_find_active_localhost("ipv4.tcpsyncookies");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -567,7 +564,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "TCP SYN Cookies"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 3100
                             , update_every
@@ -595,7 +592,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                   tcpstat.tcps_ecn_not_supported ||
                                                   netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_ecn = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv4.ecnpkts");
+                st = rrdset_find_active_localhost("ipv4.ecnpkts");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -605,7 +602,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 ECN Statistics"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 8700
                             , update_every
@@ -638,7 +635,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             error("DISABLED: ipv4.udperrors");
         } else {
             if (likely(do_udp_packets)) {
-                st = rrdset_find_localhost("ipv4.udppackets");
+                st = rrdset_find_active_localhost("ipv4.udppackets");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -648,7 +645,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 UDP Packets"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 2601
                             , update_every
@@ -668,7 +665,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             // --------------------------------------------------------------------
 
             if (likely(do_udp_errors)) {
-                st = rrdset_find_localhost("ipv4.udperrors");
+                st = rrdset_find_active_localhost("ipv4.udperrors");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -678,7 +675,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 UDP Errors"
                             , "events/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 2701
                             , update_every
@@ -729,7 +726,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             // --------------------------------------------------------------------
 
             if (likely(do_icmp_packets)) {
-                st = rrdset_find_localhost("ipv4.icmp");
+                st = rrdset_find_active_localhost("ipv4.icmp");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -739,7 +736,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 ICMP Packets"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 2602
                             , update_every
@@ -758,7 +755,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
 
                 // --------------------------------------------------------------------
 
-                st = rrdset_find_localhost("ipv4.icmp_errors");
+                st = rrdset_find_active_localhost("ipv4.icmp_errors");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -768,7 +765,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 ICMP Errors"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 2603
                             , update_every
@@ -791,7 +788,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             // --------------------------------------------------------------------
 
             if (likely(do_icmpmsg)) {
-                st = rrdset_find_localhost("ipv4.icmpmsg");
+                st = rrdset_find_active_localhost("ipv4.icmpmsg");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -801,7 +798,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 ICMP Messages"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 2604
                             , update_every
@@ -840,7 +837,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             error("DISABLED: ipv4.errors");
         } else {
             if (likely(do_ip_packets)) {
-                st = rrdset_find_localhost("ipv4.packets");
+                st = rrdset_find_active_localhost("ipv4.packets");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -850,7 +847,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 Packets"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 3000
                             , update_every
@@ -874,7 +871,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             // --------------------------------------------------------------------
 
             if (likely(do_ip_fragsout)) {
-                st = rrdset_find_localhost("ipv4.fragsout");
+                st = rrdset_find_active_localhost("ipv4.fragsout");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -884,7 +881,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 Fragments Sent"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 3010
                             , update_every
@@ -907,7 +904,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             // --------------------------------------------------------------------
 
             if (likely(do_ip_fragsin)) {
-                st = rrdset_find_localhost("ipv4.fragsin");
+                st = rrdset_find_active_localhost("ipv4.fragsin");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -917,7 +914,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 Fragments Reassembly"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 3011
                             , update_every
@@ -940,7 +937,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             // --------------------------------------------------------------------
 
             if (likely(do_ip_errors)) {
-                st = rrdset_find_localhost("ipv4.errors");
+                st = rrdset_find_active_localhost("ipv4.errors");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv4"
@@ -950,7 +947,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv4 Errors"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 3002
                             , update_every
@@ -1000,7 +997,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                           ip6stat.ip6s_delivered ||
                                                           netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_ip6_packets = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv6.packets");
+                st = rrdset_find_active_localhost("ipv6.packets");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv6"
@@ -1010,7 +1007,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv6 Packets"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 3000
                             , update_every
@@ -1039,7 +1036,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                            ip6stat.ip6s_ofragments ||
                                                            netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_ip6_fragsout = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv6.fragsout");
+                st = rrdset_find_active_localhost("ipv6.fragsout");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv6"
@@ -1049,7 +1046,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv6 Fragments Sent"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 3010
                             , update_every
@@ -1078,7 +1075,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                           ip6stat.ip6s_fragments ||
                                                           netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_ip6_fragsin = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv6.fragsin");
+                st = rrdset_find_active_localhost("ipv6.fragsin");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv6"
@@ -1088,7 +1085,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv6 Fragments Reassembly"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 3011
                             , update_every
@@ -1124,7 +1121,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                          ip6stat.ip6s_noroute ||
                                                          netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_ip6_errors = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv6.errors");
+                st = rrdset_find_active_localhost("ipv6.errors");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv6"
@@ -1134,7 +1131,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv6 Errors"
                             , "packets/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 3002
                             , update_every
@@ -1186,7 +1183,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                     icmp6_total.msgs_out ||
                                                     netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_icmp6 = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv6.icmp");
+                st = rrdset_find_active_localhost("ipv6.icmp");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv6"
@@ -1196,7 +1193,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv6 ICMP Messages"
                             , "messages/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 10000
                             , update_every
@@ -1220,7 +1217,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                           icmp6stat.icp6s_outhist[ND_REDIRECT] ||
                                                           netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_icmp6_redir = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv6.icmpredir");
+                st = rrdset_find_active_localhost("ipv6.icmpredir");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv6"
@@ -1230,7 +1227,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv6 ICMP Redirects"
                             , "redirects/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 10050
                             , update_every
@@ -1263,7 +1260,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                            icmp6stat.icp6s_outhist[ICMP6_PARAM_PROB] ||
                                                            netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_icmp6_errors = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv6.icmperrors");
+                st = rrdset_find_active_localhost("ipv6.icmperrors");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv6"
@@ -1273,7 +1270,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv6 ICMP Errors"
                             , "errors/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 10100
                             , update_every
@@ -1316,7 +1313,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                           icmp6stat.icp6s_outhist[ICMP6_ECHO_REPLY] ||
                                                           netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_icmp6_echos = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv6.icmpechos");
+                st = rrdset_find_active_localhost("ipv6.icmpechos");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv6"
@@ -1326,7 +1323,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv6 ICMP Echo"
                             , "messages/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 10200
                             , update_every
@@ -1356,7 +1353,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                            icmp6stat.icp6s_outhist[ND_ROUTER_ADVERT] ||
                                                            netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_icmp6_router = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv6.icmprouter");
+                st = rrdset_find_active_localhost("ipv6.icmprouter");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv6"
@@ -1366,7 +1363,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv6 Router Messages"
                             , "messages/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 10400
                             , update_every
@@ -1396,7 +1393,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                              icmp6stat.icp6s_outhist[ND_NEIGHBOR_ADVERT] ||
                                                              netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_icmp6_neighbor = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv6.icmpneighbor");
+                st = rrdset_find_active_localhost("ipv6.icmpneighbor");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv6"
@@ -1406,7 +1403,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv6 Neighbor Messages"
                             , "messages/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 10500
                             , update_every
@@ -1442,7 +1439,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                                                           icmp6stat.icp6s_outhist[136] ||
                                                           netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES))) {
                 do_icmp6_types = CONFIG_BOOLEAN_YES;
-                st = rrdset_find_localhost("ipv6.icmptypes");
+                st = rrdset_find_active_localhost("ipv6.icmptypes");
                 if (unlikely(!st)) {
                     st = rrdset_create_localhost(
                             "ipv6"
@@ -1452,7 +1449,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                             , NULL
                             , "IPv6 ICMP Types"
                             , "messages/s"
-                            , "macos"
+                            , "macos.plugin"
                             , "sysctl"
                             , 10700
                             , update_every
@@ -1495,7 +1492,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
             error("DISABLED: system.uptime");
         } else {
             clock_gettime(CLOCK_REALTIME, &cur_time);
-            st = rrdset_find_localhost("system.uptime");
+            st = rrdset_find_active_localhost("system.uptime");
 
             if(unlikely(!st)) {
                 st = rrdset_create_localhost(
@@ -1506,7 +1503,7 @@ int do_macos_sysctl(int update_every, usec_t dt) {
                         , NULL
                         , "System Uptime"
                         , "seconds"
-                        , "macos"
+                        , "macos.plugin"
                         , "sysctl"
                         , 1000
                         , update_every
