@@ -38,14 +38,18 @@ struct rrdcalc {
 
     char *name;                     // the name of this alarm
     uint32_t hash;                  // the hash of the alarm name
+    uuid_t config_hash_id;          // a predictable hash_id based on specific alert configuration
 
     char *exec;                     // the command to execute when this alarm switches state
     char *recipient;                // the recipient of the alarm (the first parameter to exec)
 
+    char *classification;           // the class that this alarm belongs
+    char *component;                // the component that this alarm refers to
+    char *type;                     // type of the alarm
+
     char *chart;                    // the chart id this should be linked to
     uint32_t hash_chart;
 
-
     char *plugin_match;             //the plugin name that should be linked to
     SIMPLE_PATTERN *plugin_pattern;
 
@@ -117,6 +121,7 @@ struct rrdcalc {
     time_t next_update;             // the next update timestamp of the alarm
     time_t last_status_change;      // the timestamp of the last time this alarm changed status
     time_t last_repeat; // the last time the alarm got repeated
+    uint32_t times_repeat;          // number of times the alarm got repeated
 
     time_t db_after;                // the first timestamp evaluated by the db lookup
     time_t db_before;               // the last timestamp evaluated by the db lookup
@@ -146,6 +151,43 @@ struct rrdcalc {
     struct rrdcalc *next;
 };
 
+struct alert_config {
+    char *alarm;
+    char *template_key;
+    char *os;
+    char *host;
+    char *on;
+    char *families;
+    char *plugin;
+    char *module;
+    char *charts;
+    char *lookup;
+    char *calc;
+    char *warn;
+    char *crit;
+    char *every;
+    char *green;
+    char *red;
+    char *exec;
+    char *to;
+    char *units;
+    char *info;
+    char *classification;
+    char *component;
+    char *type;
+    char *delay;
+    char *options;
+    char *repeat;
+    char *host_labels;
+
+    char *p_db_lookup_dimensions;
+    char *p_db_lookup_method;
+    uint32_t p_db_lookup_options;
+    int32_t p_db_lookup_after;
+    int32_t p_db_lookup_before;
+    int32_t p_update_every;
+};
+
 extern int alarm_isrepeating(RRDHOST *host, uint32_t alarm_id);
 extern int alarm_entry_isrepeating(RRDHOST *host, ALARM_ENTRY *ae);
 extern RRDCALC *alarm_max_last_repeat(RRDHOST *host, char *alarm_name, uint32_t hash);
