@@ -7,29 +7,46 @@
 // PROMETHEUS
 // /api/v1/allmetrics?format=prometheus and /api/v1/allmetrics?format=prometheus_all_hosts
 
+static int is_matches_rrdset(struct instance *instance, RRDSET *st, SIMPLE_PATTERN *filter) {
+    if (instance->config.options & EXPORTING_OPTION_SEND_NAMES) {
+        return simple_pattern_matches(filter, st->name);
+    }
+    return simple_pattern_matches(filter, st->id);
+}
+
 /**
  * Check if a chart can be sent to Prometheus
  *
  * @param instance an instance data structure.
  * @param st a chart.
+ * @param filter a simple pattern to match against.
  * @return Returns 1 if the chart can be sent, 0 otherwise.
  */
-inline int can_send_rrdset(struct instance *instance, RRDSET *st)
+inline int can_send_rrdset(struct instance *instance, RRDSET *st, SIMPLE_PATTERN *filter)
 {
+#ifdef NETDATA_INTERNAL_CHECKS
     RRDHOST *host = st->rrdhost;
+#endif
 
-    if (unlikely(rrdset_flag_check(st, RRDSET_FLAG_BACKEND_IGNORE)))
+    // Do not send anomaly rates charts.
+    if (st->state && st->state->is_ar_chart)
         return 0;
 
-    if (unlikely(!rrdset_flag_check(st, RRDSET_FLAG_BACKEND_SEND))) {
+    if (unlikely(rrdset_flag_check(st, RRDSET_FLAG_EXPORTING_IGNORE)))
+        return 0;
+
+    if (filter) {
+        if (!is_matches_rrdset(instance, st, filter)) {
+            return 0;
+        }
+    } else if (unlikely(!rrdset_flag_check(st, RRDSET_FLAG_EXPORTING_SEND))) {
         // we have not checked this chart
-        if (simple_pattern_matches(instance->config.charts_pattern, st->id) ||
-            simple_pattern_matches(instance->config.charts_pattern, st->name))
-            rrdset_flag_set(st, RRDSET_FLAG_BACKEND_SEND);
-        else {
-            rrdset_flag_set(st, RRDSET_FLAG_BACKEND_IGNORE);
+        if (is_matches_rrdset(instance, st, instance->config.charts_pattern)) {
+            rrdset_flag_set(st, RRDSET_FLAG_EXPORTING_SEND);
+        } else {
+            rrdset_flag_set(st, RRDSET_FLAG_EXPORTING_IGNORE);
             debug(
-                D_BACKEND,
+                D_EXPORTING,
                 "EXPORTING: not sending chart '%s' of host '%s', because it is disabled for exporting.",
                 st->id,
                 host->hostname);
@@ -39,7 +56,7 @@ inline int can_send_rrdset(struct instance *instance, RRDSET *st)
 
     if (unlikely(!rrdset_is_available_for_exporting_and_alarms(st))) {
         debug(
-            D_BACKEND,
+            D_EXPORTING,
             "EXPORTING: not sending chart '%s' of host '%s', because it is not available for exporting.",
             st->id,
             host->hostname);
@@ -50,7 +67,7 @@ inline int can_send_rrdset(struct instance *instance, RRDSET *st)
             st->rrd_memory_mode == RRD_MEMORY_MODE_NONE &&
             !(EXPORTING_OPTIONS_DATA_SOURCE(instance->config.options) == EXPORTING_SOURCE_DATA_AS_COLLECTED))) {
         debug(
-            D_BACKEND,
+            D_EXPORTING,
             "EXPORTING: not sending chart '%s' of host '%s' because its memory mode is '%s' and the exporting connector requires database access.",
             st->id,
             host->hostname,
@@ -136,7 +153,7 @@ static inline time_t prometheus_server_last_access(const char *server, RRDHOST *
  * Copy and sanitize name.
  *
  * @param d a destination string.
- * @param s a source sting.
+ * @param s a source string.
  * @param usable the number of characters to copy.
  * @return Returns the length of the copied string.
  */
@@ -161,7 +178,7 @@ inline size_t prometheus_name_copy(char *d, const char *s, size_t usable)
  * Copy and sanitize label.
  *
  * @param d a destination string.
- * @param s a source sting.
+ * @param s a source string.
  * @param usable the number of characters to copy.
  * @return Returns the length of the copied string.
  */
@@ -190,7 +207,7 @@ inline size_t prometheus_label_copy(char *d, const char *s, size_t usable)
  * Copy and sanitize units.
  *
  * @param d a destination string.
- * @param s a source sting.
+ * @param s a source string.
  * @param usable the number of characters to copy.
  * @param showoldunits set this flag to 1 to show old (before v1.12) units.
  * @return Returns the destination string.
@@ -474,6 +491,7 @@ static void generate_as_collected_prom_metric(BUFFER *wb, struct gen_parameters
  *
  * @param instance an instance data structure.
  * @param host a data collecting host.
+ * @param filter_string a simple pattern filter.
  * @param wb the buffer to fill with metrics.
  * @param prefix a prefix for every metric.
  * @param exporting_options options to configure what data is exported.
@@ -483,12 +501,14 @@ static void generate_as_collected_prom_metric(BUFFER *wb, struct gen_parameters
 static void rrd_stats_api_v1_charts_allmetrics_prometheus(
     struct instance *instance,
     RRDHOST *host,
+    const char *filter_string,
     BUFFER *wb,
     const char *prefix,
     EXPORTING_OPTIONS exporting_options,
     int allhosts,
     PROMETHEUS_OUTPUT_OPTIONS output_options)
 {
+    SIMPLE_PATTERN *filter = simple_pattern_create(filter_string, NULL, SIMPLE_PATTERN_EXACT);
     rrdhost_rdlock(host);
 
     char hostname[PROMETHEUS_ELEMENT_MAX + 1];
@@ -586,7 +606,7 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
     rrdset_foreach_read(st, host)
     {
 
-        if (likely(can_send_rrdset(instance, st))) {
+        if (likely(can_send_rrdset(instance, st, filter))) {
             rrdset_rdlock(st);
 
             char chart[PROMETHEUS_ELEMENT_MAX + 1];
@@ -771,6 +791,7 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
     }
 
     rrdhost_unlock(host);
+    simple_pattern_free(filter);
 }
 
 /**
@@ -794,6 +815,9 @@ static inline time_t prometheus_preparation(
     time_t now,
     PROMETHEUS_OUTPUT_OPTIONS output_options)
 {
+#ifndef UNIT_TESTING
+    analytics_log_prometheus();
+#endif
     if (!server || !*server)
         server = "default";
 
@@ -841,6 +865,7 @@ static inline time_t prometheus_preparation(
  * Write metrics and auxiliary information for one host to a buffer.
  *
  * @param host a data collecting host.
+ * @param filter_string a simple pattern filter.
  * @param wb the buffer to write to.
  * @param server the name of a Prometheus server.
  * @param prefix a prefix for every metric.
@@ -849,13 +874,14 @@ static inline time_t prometheus_preparation(
  */
 void rrd_stats_api_v1_charts_allmetrics_prometheus_single_host(
     RRDHOST *host,
+    const char *filter_string,
     BUFFER *wb,
     const char *server,
     const char *prefix,
     EXPORTING_OPTIONS exporting_options,
     PROMETHEUS_OUTPUT_OPTIONS output_options)
 {
-    if (unlikely(!prometheus_exporter_instance))
+    if (unlikely(!prometheus_exporter_instance || !prometheus_exporter_instance->config.initialized))
         return;
 
     prometheus_exporter_instance->before = now_realtime_sec();
@@ -871,13 +897,14 @@ void rrd_stats_api_v1_charts_allmetrics_prometheus_single_host(
         output_options);
 
     rrd_stats_api_v1_charts_allmetrics_prometheus(
-        prometheus_exporter_instance, host, wb, prefix, exporting_options, 0, output_options);
+        prometheus_exporter_instance, host, filter_string, wb, prefix, exporting_options, 0, output_options);
 }
 
 /**
  * Write metrics and auxiliary information for all hosts to a buffer.
  *
  * @param host a data collecting host.
+ * @param filter_string a simple pattern filter.
  * @param wb the buffer to write to.
  * @param server the name of a Prometheus server.
  * @param prefix a prefix for every metric.
@@ -886,13 +913,14 @@ void rrd_stats_api_v1_charts_allmetrics_prometheus_single_host(
  */
 void rrd_stats_api_v1_charts_allmetrics_prometheus_all_hosts(
     RRDHOST *host,
+    const char *filter_string,
     BUFFER *wb,
     const char *server,
     const char *prefix,
     EXPORTING_OPTIONS exporting_options,
     PROMETHEUS_OUTPUT_OPTIONS output_options)
 {
-    if (unlikely(!prometheus_exporter_instance))
+    if (unlikely(!prometheus_exporter_instance || !prometheus_exporter_instance->config.initialized))
         return;
 
     prometheus_exporter_instance->before = now_realtime_sec();
@@ -911,7 +939,7 @@ void rrd_stats_api_v1_charts_allmetrics_prometheus_all_hosts(
     rrdhost_foreach_read(host)
     {
         rrd_stats_api_v1_charts_allmetrics_prometheus(
-            prometheus_exporter_instance, host, wb, prefix, exporting_options, 1, output_options);
+            prometheus_exporter_instance, host, filter_string, wb, prefix, exporting_options, 1, output_options);
     }
     rrd_unlock();
 }
