@@ -10,7 +10,7 @@ netdata_rwlock_t rrd_rwlock;
 struct config netdata_config;
 char *netdata_configured_user_config_dir = ".";
 char *netdata_configured_stock_config_dir = ".";
-char *netdata_configured_hostname = "test_host";
+char *netdata_configured_hostname = "test_global_host";
 
 char log_line[MAX_LOG_LINE + 1];
 
@@ -77,7 +77,7 @@ static void test_read_exporting_config(void **state)
     *state = engine;
 
     assert_ptr_not_equal(engine, NULL);
-    assert_string_equal(engine->config.hostname, "test-host");
+    assert_string_equal(engine->config.hostname, "test_engine_host");
     assert_int_equal(engine->config.update_every, 3);
     assert_int_equal(engine->instance_num, 0);
 
@@ -122,7 +122,6 @@ static void test_init_connectors(void **state)
     assert_ptr_equal(instance->metric_formatting, format_dimension_collected_graphite_plaintext);
     assert_ptr_equal(instance->end_chart_formatting, NULL);
     assert_ptr_equal(instance->end_host_formatting, flush_host_labels);
-    assert_ptr_equal(instance->end_batch_formatting, simple_connector_update_buffered_bytes);
 
     BUFFER *buffer = instance->buffer;
     assert_ptr_not_equal(buffer, NULL);
@@ -231,7 +230,7 @@ static void test_rrdhost_is_exportable(void **state)
     assert_string_equal(log_line, "enabled exporting of host 'localhost' for instance 'instance_name'");
 
     assert_ptr_not_equal(localhost->exporting_flags, NULL);
-    assert_int_equal(localhost->exporting_flags[0], RRDHOST_FLAG_BACKEND_SEND);
+    assert_int_equal(localhost->exporting_flags[0], RRDHOST_FLAG_EXPORTING_SEND);
 }
 
 static void test_false_rrdhost_is_exportable(void **state)
@@ -251,7 +250,7 @@ static void test_false_rrdhost_is_exportable(void **state)
     assert_string_equal(log_line, "disabled exporting of host 'localhost' for instance 'instance_name'");
 
     assert_ptr_not_equal(localhost->exporting_flags, NULL);
-    assert_int_equal(localhost->exporting_flags[0], RRDHOST_FLAG_BACKEND_DONT_SEND);
+    assert_int_equal(localhost->exporting_flags[0], RRDHOST_FLAG_EXPORTING_DONT_SEND);
 }
 
 static void test_rrdset_is_exportable(void **state)
@@ -265,7 +264,7 @@ static void test_rrdset_is_exportable(void **state)
     assert_int_equal(__real_rrdset_is_exportable(instance, st), 1);
 
     assert_ptr_not_equal(st->exporting_flags, NULL);
-    assert_int_equal(st->exporting_flags[0], RRDSET_FLAG_BACKEND_SEND);
+    assert_int_equal(st->exporting_flags[0], RRDSET_FLAG_EXPORTING_SEND);
 }
 
 static void test_false_rrdset_is_exportable(void **state)
@@ -282,7 +281,7 @@ static void test_false_rrdset_is_exportable(void **state)
     assert_int_equal(__real_rrdset_is_exportable(instance, st), 0);
 
     assert_ptr_not_equal(st->exporting_flags, NULL);
-    assert_int_equal(st->exporting_flags[0], RRDSET_FLAG_BACKEND_IGNORE);
+    assert_int_equal(st->exporting_flags[0], RRDSET_FLAG_EXPORTING_IGNORE);
 }
 
 static void test_exporting_calculate_value_from_stored_data(void **state)
@@ -308,12 +307,12 @@ static void test_exporting_calculate_value_from_stored_data(void **state)
     expect_function_call(__mock_rrddim_query_is_finished);
     will_return(__mock_rrddim_query_is_finished, 0);
     expect_function_call(__mock_rrddim_query_next_metric);
-    will_return(__mock_rrddim_query_next_metric, pack_storage_number(27, SN_EXISTS));
+    will_return(__mock_rrddim_query_next_metric, pack_storage_number(27, SN_DEFAULT_FLAGS));
 
     expect_function_call(__mock_rrddim_query_is_finished);
     will_return(__mock_rrddim_query_is_finished, 0);
     expect_function_call(__mock_rrddim_query_next_metric);
-    will_return(__mock_rrddim_query_next_metric, pack_storage_number(45, SN_EXISTS));
+    will_return(__mock_rrddim_query_next_metric, pack_storage_number(45, SN_DEFAULT_FLAGS));
 
     expect_function_call(__mock_rrddim_query_is_finished);
     will_return(__mock_rrddim_query_is_finished, 1);
@@ -427,7 +426,7 @@ static void test_format_dimension_stored_graphite_plaintext(void **state)
     struct engine *engine = *state;
 
     expect_function_call(__wrap_exporting_calculate_value_from_stored_data);
-    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_EXISTS));
+    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_DEFAULT_FLAGS));
 
     RRDDIM *rd = localhost->rrdset_root->dimensions;
     assert_int_equal(format_dimension_stored_graphite_plaintext(engine->instance_root, rd), 0);
@@ -455,7 +454,7 @@ static void test_format_dimension_stored_json_plaintext(void **state)
     struct engine *engine = *state;
 
     expect_function_call(__wrap_exporting_calculate_value_from_stored_data);
-    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_EXISTS));
+    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_DEFAULT_FLAGS));
 
     RRDDIM *rd = localhost->rrdset_root->dimensions;
     assert_int_equal(format_dimension_stored_json_plaintext(engine->instance_root, rd), 0);
@@ -483,7 +482,7 @@ static void test_format_dimension_stored_opentsdb_telnet(void **state)
     struct engine *engine = *state;
 
     expect_function_call(__wrap_exporting_calculate_value_from_stored_data);
-    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_EXISTS));
+    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_DEFAULT_FLAGS));
 
     RRDDIM *rd = localhost->rrdset_root->dimensions;
     assert_int_equal(format_dimension_stored_opentsdb_telnet(engine->instance_root, rd), 0);
@@ -500,14 +499,10 @@ static void test_format_dimension_collected_opentsdb_http(void **state)
     assert_int_equal(format_dimension_collected_opentsdb_http(engine->instance_root, rd), 0);
     assert_string_equal(
         buffer_tostring(engine->instance_root->buffer),
-        "POST /api/put HTTP/1.1\r\n"
-        "Host: test-host\r\n"
-        "Content-Type: application/json\r\n"
-        "Content-Length: 153\r\n\r\n"
-        "{  \"metric\": \"netdata.chart_name.dimension_name\",  "
-        "\"timestamp\": 15051,  "
-        "\"value\": 123000321,  "
-        "\"tags\": {    \"host\": \"test-host TAG1=VALUE1 TAG2=VALUE2\"  }}");
+        "{\"metric\":\"netdata.chart_name.dimension_name\","
+        "\"timestamp\":15051,"
+        "\"value\":123000321,"
+        "\"tags\":{\"host\":\"test-host TAG1=VALUE1 TAG2=VALUE2\"}}");
 }
 
 static void test_format_dimension_stored_opentsdb_http(void **state)
@@ -515,20 +510,16 @@ static void test_format_dimension_stored_opentsdb_http(void **state)
     struct engine *engine = *state;
 
     expect_function_call(__wrap_exporting_calculate_value_from_stored_data);
-    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_EXISTS));
+    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_DEFAULT_FLAGS));
 
     RRDDIM *rd = localhost->rrdset_root->dimensions;
     assert_int_equal(format_dimension_stored_opentsdb_http(engine->instance_root, rd), 0);
     assert_string_equal(
         buffer_tostring(engine->instance_root->buffer),
-        "POST /api/put HTTP/1.1\r\n"
-        "Host: test-host\r\n"
-        "Content-Type: application/json\r\n"
-        "Content-Length: 161\r\n\r\n"
-        "{  \"metric\": \"netdata.chart_name.dimension_name\",  "
-        "\"timestamp\": 15052,  "
-        "\"value\": 690565856.0000000,  "
-        "\"tags\": {    \"host\": \"test-host TAG1=VALUE1 TAG2=VALUE2\"  }}");
+        "{\"metric\":\"netdata.chart_name.dimension_name\","
+        "\"timestamp\":15052,"
+        "\"value\":690565856.0000000,"
+        "\"tags\":{\"host\":\"test-host TAG1=VALUE1 TAG2=VALUE2\"}}");
 }
 
 static void test_exporting_discard_response(void **state)
@@ -538,14 +529,7 @@ static void test_exporting_discard_response(void **state)
     BUFFER *response = buffer_create(0);
     buffer_sprintf(response, "Test response");
 
-    expect_function_call(__wrap_info_int);
-
     assert_int_equal(exporting_discard_response(response, engine->instance_root), 0);
-
-    assert_string_equal(
-        log_line,
-        "EXPORTING: received 13 bytes from instance_name connector instance. Ignoring them. Sample: 'Test response'");
-
     assert_int_equal(buffer_strlen(response), 0);
 
     buffer_free(response);
@@ -565,14 +549,8 @@ static void test_simple_connector_receive_response(void **state)
     expect_value(__wrap_recv, len, 4096);
     expect_value(__wrap_recv, flags, MSG_DONTWAIT);
 
-    expect_function_call(__wrap_info_int);
-
     simple_connector_receive_response(&sock, instance);
 
-    assert_string_equal(
-        log_line,
-        "EXPORTING: received 9 bytes from instance_name connector instance. Ignoring them. Sample: 'Test recv'");
-
     assert_int_equal(stats->received_bytes, 9);
     assert_int_equal(stats->receptions, 1);
     assert_int_equal(sock, 1);
@@ -583,39 +561,34 @@ static void test_simple_connector_send_buffer(void **state)
     struct engine *engine = *state;
     struct instance *instance = engine->instance_root;
     struct stats *stats = &instance->stats;
-    BUFFER *buffer = instance->buffer;
 
     int sock = 1;
     int failures = 3;
+    size_t buffered_metrics = 1;
+    BUFFER *header = buffer_create(0);
+    BUFFER *buffer = buffer_create(0);
+    buffer_strcat(header, "test header\n");
+    buffer_strcat(buffer, "test buffer\n");
 
-    __real_mark_scheduled_instances(engine);
-
-    expect_function_call(__wrap_rrdhost_is_exportable);
-    expect_value(__wrap_rrdhost_is_exportable, instance, instance);
-    expect_value(__wrap_rrdhost_is_exportable, host, localhost);
-    will_return(__wrap_rrdhost_is_exportable, 1);
-
-    RRDSET *st = localhost->rrdset_root;
-    expect_function_call(__wrap_rrdset_is_exportable);
-    expect_value(__wrap_rrdset_is_exportable, instance, instance);
-    expect_value(__wrap_rrdset_is_exportable, st, st);
-    will_return(__wrap_rrdset_is_exportable, 1);
-
-    __real_prepare_buffers(engine);
+    expect_function_call(__wrap_send);
+    expect_value(__wrap_send, sockfd, 1);
+    expect_value(__wrap_send, buf, buffer_tostring(header));
+    expect_string(__wrap_send, buf, "test header\n");
+    expect_value(__wrap_send, len, 12);
+    expect_value(__wrap_send, flags, MSG_NOSIGNAL);
 
     expect_function_call(__wrap_send);
     expect_value(__wrap_send, sockfd, 1);
     expect_value(__wrap_send, buf, buffer_tostring(buffer));
-    expect_string(
-        __wrap_send, buf, "netdata.test-host.chart_name.dimension_name;TAG1=VALUE1 TAG2=VALUE2 123000321 15051\n");
-    expect_value(__wrap_send, len, 84);
+    expect_string(__wrap_send, buf, "test buffer\n");
+    expect_value(__wrap_send, len, 12);
     expect_value(__wrap_send, flags, MSG_NOSIGNAL);
 
-    simple_connector_send_buffer(&sock, &failures, instance);
+    simple_connector_send_buffer(&sock, &failures, instance, header, buffer, buffered_metrics);
 
     assert_int_equal(failures, 0);
     assert_int_equal(stats->transmission_successes, 1);
-    assert_int_equal(stats->sent_bytes, 84);
+    assert_int_equal(stats->sent_bytes, 12);
     assert_int_equal(stats->sent_metrics, 1);
     assert_int_equal(stats->transmission_failures, 0);
 
@@ -629,37 +602,42 @@ static void test_simple_connector_worker(void **state)
     struct engine *engine = *state;
     struct instance *instance = engine->instance_root;
     struct stats *stats = &instance->stats;
-    BUFFER *buffer = instance->buffer;
 
     __real_mark_scheduled_instances(engine);
 
-    expect_function_call(__wrap_rrdhost_is_exportable);
-    expect_value(__wrap_rrdhost_is_exportable, instance, instance);
-    expect_value(__wrap_rrdhost_is_exportable, host, localhost);
-    will_return(__wrap_rrdhost_is_exportable, 1);
+    struct simple_connector_data *simple_connector_data = callocz(1, sizeof(struct simple_connector_data));
+    instance->connector_specific_data = simple_connector_data;
+    simple_connector_data->last_buffer = callocz(1, sizeof(struct simple_connector_buffer));
+    simple_connector_data->first_buffer = simple_connector_data->last_buffer;
+    simple_connector_data->header = buffer_create(0);
+    simple_connector_data->buffer = buffer_create(0);
+    simple_connector_data->last_buffer->header = buffer_create(0);
+    simple_connector_data->last_buffer->buffer = buffer_create(0);
+    strcpy(simple_connector_data->connected_to, "localhost");
 
-    RRDSET *st = localhost->rrdset_root;
-    expect_function_call(__wrap_rrdset_is_exportable);
-    expect_value(__wrap_rrdset_is_exportable, instance, instance);
-    expect_value(__wrap_rrdset_is_exportable, st, st);
-    will_return(__wrap_rrdset_is_exportable, 1);
-
-    __real_prepare_buffers(engine);
+    buffer_sprintf(simple_connector_data->last_buffer->header, "test header");
+    buffer_sprintf(simple_connector_data->last_buffer->buffer, "test buffer");
 
     expect_function_call(__wrap_connect_to_one_of);
     expect_string(__wrap_connect_to_one_of, destination, "localhost");
     expect_value(__wrap_connect_to_one_of, default_port, 2003);
     expect_not_value(__wrap_connect_to_one_of, reconnects_counter, 0);
-    expect_value(__wrap_connect_to_one_of, connected_to, 0);
-    expect_value(__wrap_connect_to_one_of, connected_to_size, 0);
+    expect_string(__wrap_connect_to_one_of, connected_to, "localhost");
+    expect_value(__wrap_connect_to_one_of, connected_to_size, CONNECTED_TO_MAX);
     will_return(__wrap_connect_to_one_of, 2);
 
     expect_function_call(__wrap_send);
     expect_value(__wrap_send, sockfd, 2);
-    expect_value(__wrap_send, buf, buffer_tostring(buffer));
-    expect_string(
-        __wrap_send, buf, "netdata.test-host.chart_name.dimension_name;TAG1=VALUE1 TAG2=VALUE2 123000321 15051\n");
-    expect_value(__wrap_send, len, 84);
+    expect_not_value(__wrap_send, buf, buffer_tostring(simple_connector_data->last_buffer->buffer));
+    expect_string(__wrap_send, buf, "test header");
+    expect_value(__wrap_send, len, 11);
+    expect_value(__wrap_send, flags, MSG_NOSIGNAL);
+
+    expect_function_call(__wrap_send);
+    expect_value(__wrap_send, sockfd, 2);
+    expect_value(__wrap_send, buf, buffer_tostring(simple_connector_data->last_buffer->buffer));
+    expect_string(__wrap_send, buf, "test buffer");
+    expect_value(__wrap_send, len, 11);
     expect_value(__wrap_send, flags, MSG_NOSIGNAL);
 
     expect_function_call(__wrap_send_internal_metrics);
@@ -669,13 +647,13 @@ static void test_simple_connector_worker(void **state)
     simple_connector_worker(instance);
 
     assert_int_equal(stats->buffered_metrics, 0);
-    assert_int_equal(stats->buffered_bytes, 84);
+    assert_int_equal(stats->buffered_bytes, 0);
     assert_int_equal(stats->received_bytes, 0);
-    assert_int_equal(stats->sent_bytes, 84);
-    assert_int_equal(stats->sent_metrics, 1);
+    assert_int_equal(stats->sent_bytes, 0);
+    assert_int_equal(stats->sent_metrics, 0);
     assert_int_equal(stats->lost_metrics, 0);
     assert_int_equal(stats->receptions, 0);
-    assert_int_equal(stats->transmission_successes, 1);
+    assert_int_equal(stats->transmission_successes, 0);
     assert_int_equal(stats->transmission_failures, 0);
     assert_int_equal(stats->data_lost_events, 0);
     assert_int_equal(stats->lost_bytes, 0);
@@ -1010,21 +988,21 @@ static void test_can_send_rrdset(void **state)
 {
     (void)*state;
 
-    assert_int_equal(can_send_rrdset(prometheus_exporter_instance, localhost->rrdset_root), 1);
+    assert_int_equal(can_send_rrdset(prometheus_exporter_instance, localhost->rrdset_root, NULL), 1);
 
-    rrdset_flag_set(localhost->rrdset_root, RRDSET_FLAG_BACKEND_IGNORE);
-    assert_int_equal(can_send_rrdset(prometheus_exporter_instance, localhost->rrdset_root), 0);
-    rrdset_flag_clear(localhost->rrdset_root, RRDSET_FLAG_BACKEND_IGNORE);
+    rrdset_flag_set(localhost->rrdset_root, RRDSET_FLAG_EXPORTING_IGNORE);
+    assert_int_equal(can_send_rrdset(prometheus_exporter_instance, localhost->rrdset_root, NULL), 0);
+    rrdset_flag_clear(localhost->rrdset_root, RRDSET_FLAG_EXPORTING_IGNORE);
 
     // TODO: test with a denying simple pattern
 
     rrdset_flag_set(localhost->rrdset_root, RRDSET_FLAG_OBSOLETE);
-    assert_int_equal(can_send_rrdset(prometheus_exporter_instance, localhost->rrdset_root), 0);
+    assert_int_equal(can_send_rrdset(prometheus_exporter_instance, localhost->rrdset_root, NULL), 0);
     rrdset_flag_clear(localhost->rrdset_root, RRDSET_FLAG_OBSOLETE);
 
     localhost->rrdset_root->rrd_memory_mode = RRD_MEMORY_MODE_NONE;
     prometheus_exporter_instance->config.options |= EXPORTING_SOURCE_DATA_AVERAGE;
-    assert_int_equal(can_send_rrdset(prometheus_exporter_instance, localhost->rrdset_root), 0);
+    assert_int_equal(can_send_rrdset(prometheus_exporter_instance, localhost->rrdset_root, NULL), 0);
 }
 
 static void test_prometheus_name_copy(void **state)
@@ -1070,7 +1048,7 @@ static void test_format_host_labels_prometheus(void **state)
     instance->config.options |= EXPORTING_OPTION_SEND_AUTOMATIC_LABELS;
 
     format_host_labels_prometheus(instance, localhost);
-    assert_string_equal(buffer_tostring(instance->labels), "key1=\"netdata\",key2=\"value2\"");
+    assert_string_equal(buffer_tostring(instance->labels), "key1=\"value1\",key2=\"value2\"");
 }
 
 static void rrd_stats_api_v1_charts_allmetrics_prometheus(void **state)
@@ -1087,9 +1065,9 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(void **state)
     will_return(__wrap_now_realtime_sec, 2);
 
     expect_function_call(__wrap_exporting_calculate_value_from_stored_data);
-    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_EXISTS));
+    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_DEFAULT_FLAGS));
 
-    rrd_stats_api_v1_charts_allmetrics_prometheus_single_host(localhost, buffer, "test_server", "test_prefix", 0, 0);
+    rrd_stats_api_v1_charts_allmetrics_prometheus_single_host(localhost, NULL, buffer, "test_server", "test_prefix", 0, 0);
 
     assert_string_equal(
         buffer_tostring(buffer),
@@ -1104,10 +1082,10 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(void **state)
     will_return(__wrap_now_realtime_sec, 2);
 
     expect_function_call(__wrap_exporting_calculate_value_from_stored_data);
-    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_EXISTS));
+    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_DEFAULT_FLAGS));
 
     rrd_stats_api_v1_charts_allmetrics_prometheus_single_host(
-        localhost, buffer, "test_server", "test_prefix", 0, PROMETHEUS_OUTPUT_NAMES | PROMETHEUS_OUTPUT_TYPES);
+        localhost, NULL, buffer, "test_server", "test_prefix", 0, PROMETHEUS_OUTPUT_NAMES | PROMETHEUS_OUTPUT_TYPES);
 
     assert_string_equal(
         buffer_tostring(buffer),
@@ -1123,9 +1101,9 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(void **state)
     will_return(__wrap_now_realtime_sec, 2);
 
     expect_function_call(__wrap_exporting_calculate_value_from_stored_data);
-    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_EXISTS));
+    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_DEFAULT_FLAGS));
 
-    rrd_stats_api_v1_charts_allmetrics_prometheus_all_hosts(localhost, buffer, "test_server", "test_prefix", 0, 0);
+    rrd_stats_api_v1_charts_allmetrics_prometheus_all_hosts(localhost, NULL, buffer, "test_server", "test_prefix", 0, 0);
 
     assert_string_equal(
         buffer_tostring(buffer),
@@ -1159,7 +1137,7 @@ static void test_init_prometheus_remote_write_instance(void **state)
     assert_ptr_equal(instance->end_chart_formatting, NULL);
     assert_ptr_equal(instance->end_host_formatting, NULL);
     assert_ptr_equal(instance->end_batch_formatting, format_batch_prometheus_remote_write);
-    assert_ptr_equal(instance->send_header, prometheus_remote_write_send_header);
+    assert_ptr_equal(instance->prepare_header, prometheus_remote_write_prepare_header);
     assert_ptr_equal(instance->check_response, process_prometheus_remote_write_response);
 
     assert_ptr_not_equal(instance->buffer, NULL);
@@ -1169,40 +1147,46 @@ static void test_init_prometheus_remote_write_instance(void **state)
         (struct prometheus_remote_write_specific_data *)instance->connector_specific_data;
 
     assert_ptr_not_equal(instance->connector_specific_data, NULL);
-    assert_ptr_equal(connector_specific_data->write_request, 0xff);
+    assert_ptr_not_equal(connector_specific_data->write_request, NULL);
     freez(instance->connector_specific_data);
 }
 
-static void test_prometheus_remote_write_send_header(void **state)
+static void test_prometheus_remote_write_prepare_header(void **state)
 {
     struct engine *engine = *state;
     struct instance *instance = engine->instance_root;
-    int sock = 1;
 
     struct prometheus_remote_write_specific_config *connector_specific_config =
         callocz(1, sizeof(struct prometheus_remote_write_specific_config));
     instance->config.connector_specific_config = connector_specific_config;
     connector_specific_config->remote_write_path = strdupz("/receive");
 
-    buffer_sprintf(instance->buffer, "test buffer");
+    struct simple_connector_data *simple_connector_data = callocz(1, sizeof(struct simple_connector_data));
+    instance->connector_specific_data = simple_connector_data;
+    simple_connector_data->last_buffer = callocz(1, sizeof(struct simple_connector_buffer));
+    simple_connector_data->last_buffer->header = buffer_create(0);
+    simple_connector_data->last_buffer->buffer = buffer_create(0);
+    strcpy(simple_connector_data->connected_to, "localhost");
 
-    expect_function_call(__wrap_send);
-    expect_value(__wrap_send, sockfd, 1);
-    expect_not_value(__wrap_send, buf, NULL);
-    expect_string(
-        __wrap_send, buf,
+    buffer_sprintf(simple_connector_data->last_buffer->buffer, "test buffer");
+
+    prometheus_remote_write_prepare_header(instance);
+
+    assert_string_equal(
+        buffer_tostring(simple_connector_data->last_buffer->header),
         "POST /receive HTTP/1.1\r\n"
-        "Host: test-host\r\n"
+        "Host: localhost\r\n"
         "Accept: */*\r\n"
+        "Content-Encoding: snappy\r\n"
+        "Content-Type: application/x-protobuf\r\n"
         "X-Prometheus-Remote-Write-Version: 0.1.0\r\n"
         "Content-Length: 11\r\n"
-        "Content-Type: application/x-www-form-urlencoded\r\n\r\n");
-    expect_value(__wrap_send, len, 167);
-    expect_value(__wrap_send, flags, MSG_NOSIGNAL);
-
-    assert_int_equal(prometheus_remote_write_send_header(&sock, instance),0);
+        "\r\n");
 
     free(connector_specific_config->remote_write_path);
+
+    buffer_free(simple_connector_data->last_buffer->header);
+    buffer_free(simple_connector_data->last_buffer->buffer);
 }
 
 static void test_process_prometheus_remote_write_response(void **state)
@@ -1224,9 +1208,11 @@ static void test_format_host_prometheus_remote_write(void **state)
     instance->config.options |= EXPORTING_OPTION_SEND_CONFIGURED_LABELS;
     instance->config.options |= EXPORTING_OPTION_SEND_AUTOMATIC_LABELS;
 
+    struct simple_connector_data *simple_connector_data = mallocz(sizeof(struct simple_connector_data *));
+    instance->connector_specific_data = simple_connector_data;
     struct prometheus_remote_write_specific_data *connector_specific_data =
         mallocz(sizeof(struct prometheus_remote_write_specific_data *));
-    instance->connector_specific_data = (void *)connector_specific_data;
+    simple_connector_data->connector_specific_data = (void *)connector_specific_data;
     connector_specific_data->write_request = (void *)0xff;
 
     localhost->program_name = strdupz("test_program");
@@ -1254,6 +1240,7 @@ static void test_format_host_prometheus_remote_write(void **state)
     assert_int_equal(format_host_prometheus_remote_write(instance, localhost), 0);
 
     freez(connector_specific_data);
+    freez(simple_connector_data);
     free(localhost->program_name);
     free(localhost->program_version);
 }
@@ -1263,15 +1250,17 @@ static void test_format_dimension_prometheus_remote_write(void **state)
     struct engine *engine = *state;
     struct instance *instance = engine->instance_root;
 
+    struct simple_connector_data *simple_connector_data = mallocz(sizeof(struct simple_connector_data *));
+    instance->connector_specific_data = simple_connector_data;
     struct prometheus_remote_write_specific_data *connector_specific_data =
         mallocz(sizeof(struct prometheus_remote_write_specific_data *));
-    instance->connector_specific_data = (void *)connector_specific_data;
+    simple_connector_data->connector_specific_data = (void *)connector_specific_data;
     connector_specific_data->write_request = (void *)0xff;
 
     RRDDIM *rd = localhost->rrdset_root->dimensions;
 
     expect_function_call(__wrap_exporting_calculate_value_from_stored_data);
-    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_EXISTS));
+    will_return(__wrap_exporting_calculate_value_from_stored_data, pack_storage_number(27, SN_DEFAULT_FLAGS));
 
     expect_function_call(__wrap_add_metric);
     expect_value(__wrap_add_metric, write_request_p, 0xff);
@@ -1291,11 +1280,16 @@ static void test_format_batch_prometheus_remote_write(void **state)
     struct engine *engine = *state;
     struct instance *instance = engine->instance_root;
 
+    struct simple_connector_data *simple_connector_data = mallocz(sizeof(struct simple_connector_data *));
+    instance->connector_specific_data = simple_connector_data;
     struct prometheus_remote_write_specific_data *connector_specific_data =
         mallocz(sizeof(struct prometheus_remote_write_specific_data *));
-    instance->connector_specific_data = (void *)connector_specific_data;
+    simple_connector_data->connector_specific_data = (void *)connector_specific_data;
     connector_specific_data->write_request = __real_init_write_request();
 
+    expect_function_call(__wrap_simple_connector_end_batch);
+    expect_value(__wrap_simple_connector_end_batch, instance, instance);
+    will_return(__wrap_simple_connector_end_batch, 0);
     __real_add_host_info(
         connector_specific_data->write_request,
         "test_name", "test_instance", "test_application", "test_version", 15051);
@@ -1310,27 +1304,65 @@ static void test_format_batch_prometheus_remote_write(void **state)
     assert_int_equal(format_batch_prometheus_remote_write(instance), 0);
 
     BUFFER *buffer = instance->buffer;
-    assert_int_equal(buffer_strlen(buffer), 192);
-
-    BUFFER *escaped_buffer = buffer_create(850);
-    size_t len = buffer_strlen(buffer);
-    char *ch = (char *)buffer_tostring(buffer);
-    for (; len > 0; ch++, len--)
-        buffer_sprintf(escaped_buffer, "\\%03o", (unsigned int)*ch);
+    char *write_request_string = calloc(1, 1000);
+    convert_write_request_to_string(buffer_tostring(buffer), buffer_strlen(buffer), write_request_string, 999);
+    assert_int_equal(strlen(write_request_string), 753);
     assert_string_equal(
-        buffer_tostring(escaped_buffer),
-        "\\37777777641\\002\\120\\012\\37777777622\\001\\012\\025\\012\\010\\137\\137\\156\\141\\155\\145\\137\\137"
-        "\\022\\011\\164\\145\\163\\164\\005\\015\\064\\012\\031\\012\\010\\151\\156\\163\\164\\141\\156\\143\\145\\022"
-        "\\015\\005\\027\\021\\017\\100\\012\\037\\012\\013\\141\\160\\160\\154\\151\\143\\141\\164\\151\\157\\156\\022"
-        "\\020\\005\\036\\035\\022\\034\\012\\027\\012\\007\\166\\145\\162\\163\\001\\035\\000\\014\\005\\035\\015\\016"
-        "\\014\\012\\026\\012\\010\\005\\020\\020\\153\\145\\171\\022\\012\\005\\012\\040\\166\\141\\154\\165\\145\\022"
-        "\\014\\011\\000\\005\\001\\030\\37777777760\\077\\020\\37777777713\\165\\012\\37777777611\\142\\37777777625"
-        "\\000\\034\\023\\012\\005\\143\\150\\141\\162\\164\\011\\075\\000\\040\\005\\014\\054\\012\\025\\012\\006\\146"
-        "\\141\\155\\151\\154\\171\\022\\013\\005\\123\\011\\015\\040\\012\\033\\012\\011\\144\\151\\155\\145\\156\\005"
-        "\\37777777607\\000\\016\\005\\032\\025\\020\\000\\012\\146\\37777777736\\000\\064\\022\\014\\011\\000\\000\\000"
-        "\\004\\130\\123\\37777777635\\101\\020\\37777777714\\165");
+        write_request_string,
+        "timeseries {\n"
+        "  labels {\n"
+        "    name: \"__name__\"\n"
+        "    value: \"test_name\"\n"
+        "  }\n"
+        "  labels {\n"
+        "    name: \"instance\"\n"
+        "    value: \"test_instance\"\n"
+        "  }\n"
+        "  labels {\n"
+        "    name: \"application\"\n"
+        "    value: \"test_application\"\n"
+        "  }\n"
+        "  labels {\n"
+        "    name: \"version\"\n"
+        "    value: \"test_version\"\n"
+        "  }\n"
+        "  labels {\n"
+        "    name: \"test_key\"\n"
+        "    value: \"test_value\"\n"
+        "  }\n"
+        "  samples {\n"
+        "    value: 1\n"
+        "    timestamp: 15051\n"
+        "  }\n"
+        "}\n"
+        "timeseries {\n"
+        "  labels {\n"
+        "    name: \"__name__\"\n"
+        "    value: \"test_name\"\n"
+        "  }\n"
+        "  labels {\n"
+        "    name: \"chart\"\n"
+        "    value: \"test chart\"\n"
+        "  }\n"
+        "  labels {\n"
+        "    name: \"family\"\n"
+        "    value: \"test_family\"\n"
+        "  }\n"
+        "  labels {\n"
+        "    name: \"dimension\"\n"
+        "    value: \"test_dimension\"\n"
+        "  }\n"
+        "  labels {\n"
+        "    name: \"instance\"\n"
+        "    value: \"test_instance\"\n"
+        "  }\n"
+        "  samples {\n"
+        "    value: 123000321\n"
+        "    timestamp: 15052\n"
+        "  }\n"
+        "}\n");
+    free(write_request_string);
 
-    buffer_free(escaped_buffer);
     protocol_buffers_shutdown();
 }
 #endif // ENABLE_PROMETHEUS_REMOTE_WRITE
@@ -1410,6 +1442,9 @@ static void test_aws_kinesis_connector_worker(void **state)
     expect_value(__wrap_rrdset_is_exportable, st, st);
     will_return(__wrap_rrdset_is_exportable, 1);
 
+    expect_function_call(__wrap_simple_connector_end_batch);
+    expect_value(__wrap_simple_connector_end_batch, instance, instance);
+    will_return(__wrap_simple_connector_end_batch, 0);
     __real_prepare_buffers(engine);
 
     struct aws_kinesis_specific_config *connector_specific_config =
@@ -1542,6 +1577,9 @@ static void test_pubsub_connector_worker(void **state)
     expect_value(__wrap_rrdset_is_exportable, st, st);
     will_return(__wrap_rrdset_is_exportable, 1);
 
+    expect_function_call(__wrap_simple_connector_end_batch);
+    expect_value(__wrap_simple_connector_end_batch, instance, instance);
+    will_return(__wrap_simple_connector_end_batch, 0);
     __real_prepare_buffers(engine);
 
     struct pubsub_specific_config *connector_specific_config =
@@ -1663,7 +1701,7 @@ static void test_init_mongodb_instance(void **state)
     assert_ptr_equal(instance->end_chart_formatting, NULL);
     assert_ptr_equal(instance->end_host_formatting, flush_host_labels);
     assert_ptr_equal(instance->end_batch_formatting, format_batch_mongodb);
-    assert_ptr_equal(instance->send_header, NULL);
+    assert_ptr_equal(instance->prepare_header, NULL);
     assert_ptr_equal(instance->check_response, NULL);
 
     assert_ptr_not_equal(instance->buffer, NULL);
@@ -1872,7 +1910,7 @@ int main(void)
         cmocka_unit_test_setup_teardown(test_prometheus_label_copy, setup_prometheus, teardown_prometheus),
         cmocka_unit_test_setup_teardown(test_prometheus_units_copy, setup_prometheus, teardown_prometheus),
         cmocka_unit_test_setup_teardown(
-            test_format_host_labels_prometheus, setup_configured_engine, teardown_configured_engine),
+            test_format_host_labels_prometheus, setup_initialized_engine, teardown_initialized_engine),
         cmocka_unit_test_setup_teardown(
             rrd_stats_api_v1_charts_allmetrics_prometheus, setup_prometheus, teardown_prometheus),
     };
@@ -1884,7 +1922,7 @@ int main(void)
         cmocka_unit_test_setup_teardown(
             test_init_prometheus_remote_write_instance, setup_configured_engine, teardown_configured_engine),
         cmocka_unit_test_setup_teardown(
-            test_prometheus_remote_write_send_header, setup_initialized_engine, teardown_initialized_engine),
+            test_prometheus_remote_write_prepare_header, setup_initialized_engine, teardown_initialized_engine),
         cmocka_unit_test(test_process_prometheus_remote_write_response),
         cmocka_unit_test_setup_teardown(
             test_format_host_prometheus_remote_write, setup_initialized_engine, teardown_initialized_engine),
