@@ -3,6 +3,7 @@
 #include "web_api_v1.h"
 
 char *api_secret;
+extern int aclk_use_new_cloud_arch;
 
 static struct {
     const char *name;
@@ -36,6 +37,7 @@ static struct {
         , {"match-names"     , 0    , RRDR_OPTION_MATCH_NAMES}
         , {"showcustomvars"  , 0    , RRDR_OPTION_CUSTOM_VARS}
         , {"allow_past"      , 0    , RRDR_OPTION_ALLOW_PAST}
+        , {"anomaly-bit"     , 0    , RRDR_OPTION_ANOMALY_BIT}
         , {                  NULL, 0, 0}
 };
 
@@ -135,16 +137,25 @@ char *get_mgmt_api_key(void) {
 
         // save it
         fd = open(api_key_filename, O_WRONLY|O_CREAT|O_TRUNC, 444);
-        if(fd == -1)
-            fatal("Cannot create unique management API key file '%s'. Please fix this.", api_key_filename);
+        if(fd == -1) {
+            error("Cannot create unique management API key file '%s'. Please adjust config parameter 'netdata management api key file' to a proper path and file.", api_key_filename);
+            goto temp_key;
+        }
 
-        if(write(fd, guid, GUID_LEN) != GUID_LEN)
-            fatal("Cannot write the unique management API key file '%s'. Please fix this.", api_key_filename);
+        if(write(fd, guid, GUID_LEN) != GUID_LEN) {
+            error("Cannot write the unique management API key file '%s'. Please adjust config parameter 'netdata management api key file' to a proper path and file with enough space left.", api_key_filename);
+            close(fd);
+            goto temp_key;
+        }
 
         close(fd);
     }
 
     return guid;
+
+temp_key:
+    info("You can still continue to use the alarm management API using the authorization token %s during this Netdata session only.", guid);
+    return guid;
 }
 
 void web_client_api_v1_management_init(void) {
@@ -399,15 +410,19 @@ inline int web_client_api_request_v1_data(RRDHOST *host, struct web_client *w, c
 
     time_t last_timestamp_in_data = 0, google_timestamp = 0;
 
-    char *chart = NULL
-    , *before_str = NULL
-    , *after_str = NULL
-    , *group_time_str = NULL
-    , *points_str = NULL
-    , *context = NULL
-    , *chart_label_key = NULL;
+    char *chart = NULL;
+    char *before_str = NULL;
+    char *after_str = NULL;
+    char *group_time_str = NULL;
+    char *points_str = NULL;
+    char *timeout_str = NULL;
+    char *max_anomaly_rates_str = NULL;
+    char *context = NULL;
+    char *chart_label_key = NULL;
+    char *chart_labels_filter = NULL;
 
     int group = RRDR_GROUPING_AVERAGE;
+    int show_dimensions = 0;
     uint32_t format = DATASOURCE_JSON;
     uint32_t options = 0x00000000;
 
@@ -426,15 +441,18 @@ inline int web_client_api_request_v1_data(RRDHOST *host, struct web_client *w, c
 
         if(!strcmp(name, "context")) context = value;
         else if(!strcmp(name, "chart_label_key")) chart_label_key = value;
+        else if(!strcmp(name, "chart_labels_filter")) chart_labels_filter = value;
         else if(!strcmp(name, "chart")) chart = value;
         else if(!strcmp(name, "dimension") || !strcmp(name, "dim") || !strcmp(name, "dimensions") || !strcmp(name, "dims")) {
             if(!dimensions) dimensions = buffer_create(100);
             buffer_strcat(dimensions, "|");
             buffer_strcat(dimensions, value);
         }
+        else if(!strcmp(name, "show_dimensions")) show_dimensions = 1;
         else if(!strcmp(name, "after")) after_str = value;
         else if(!strcmp(name, "before")) before_str = value;
         else if(!strcmp(name, "points")) points_str = value;
+        else if(!strcmp(name, "timeout")) timeout_str = value;
         else if(!strcmp(name, "gtime")) group_time_str = value;
         else if(!strcmp(name, "group")) {
             group = web_client_api_request_v1_data_group(value, RRDR_GROUPING_AVERAGE);
@@ -482,6 +500,9 @@ inline int web_client_api_request_v1_data(RRDHOST *host, struct web_client *w, c
                     outFileName = tqx_value;
             }
         }
+        else if(!strcmp(name, "max_anomaly_rates")) {
+            max_anomaly_rates_str = value;
+        }
     }
 
     // validate the google parameters given
@@ -493,6 +514,7 @@ inline int web_client_api_request_v1_data(RRDHOST *host, struct web_client *w, c
     fix_google_param(outFileName);
 
     RRDSET *st = NULL;
+    ONEWAYALLOC *owa = onewayalloc_create(0);
 
     if((!chart || !*chart) && (!context)) {
         buffer_sprintf(w->response.data, "No chart id is given at the request.");
@@ -500,22 +522,29 @@ inline int web_client_api_request_v1_data(RRDHOST *host, struct web_client *w, c
     }
 
     struct context_param  *context_param_list = NULL;
+
     if (context && !chart) {
         RRDSET *st1;
+
         uint32_t context_hash = simple_hash(context);
 
         rrdhost_rdlock(host);
+        char *words[MAX_CHART_LABELS_FILTER];
+        uint32_t hash_key_list[MAX_CHART_LABELS_FILTER];
+        int word_count = 0;
         rrdset_foreach_read(st1, host) {
             if (st1->hash_context == context_hash && !strcmp(st1->context, context) &&
-                (!chart_label_key || rrdset_contains_label_keylist(st1, chart_label_key)))
-                build_context_param_list(&context_param_list, st1);
+                (!chart_label_key || rrdset_contains_label_keylist(st1, chart_label_key)) &&
+                (!chart_labels_filter ||
+                 rrdset_matches_label_keys(st1, chart_labels_filter, words, hash_key_list, &word_count, MAX_CHART_LABELS_FILTER)))
+                    build_context_param_list(owa, &context_param_list, st1);
         }
         rrdhost_unlock(host);
         if (likely(context_param_list && context_param_list->rd))  // Just set the first one
             st = context_param_list->rd->rrdset;
         else {
-            if (!chart_label_key)
-                sql_build_context_param_list(&context_param_list, host, context, NULL);
+            if (!chart_label_key && !chart_labels_filter)
+                sql_build_context_param_list(owa, &context_param_list, host, context, NULL);
         }
     }
     else {
@@ -525,14 +554,14 @@ inline int web_client_api_request_v1_data(RRDHOST *host, struct web_client *w, c
         if (likely(st))
             st->last_accessed_time = now_realtime_sec();
         else
-            sql_build_context_param_list(&context_param_list, host, NULL, chart);
+            sql_build_context_param_list(owa, &context_param_list, host, NULL, chart);
     }
 
     if (!st) {
         if (likely(context_param_list && context_param_list->rd && context_param_list->rd->rrdset))
             st = context_param_list->rd->rrdset;
         else {
-            free_context_param_list(&context_param_list);
+            free_context_param_list(owa, &context_param_list);
             context_param_list = NULL;
         }
     }
@@ -561,7 +590,21 @@ inline int web_client_api_request_v1_data(RRDHOST *host, struct web_client *w, c
     long long before = (before_str && *before_str)?str2l(before_str):0;
     long long after  = (after_str  && *after_str) ?str2l(after_str):-600;
     int       points = (points_str && *points_str)?str2i(points_str):0;
+    int       timeout = (timeout_str && *timeout_str)?str2i(timeout_str): 0;
     long      group_time = (group_time_str && *group_time_str)?str2l(group_time_str):0;
+    int       max_anomaly_rates = (max_anomaly_rates_str && *max_anomaly_rates_str) ? str2i(max_anomaly_rates_str) : 0;
+
+    if (timeout) {
+        struct timeval now;
+        now_realtime_timeval(&now);
+        int inqueue = (int)dt_usec(&w->tv_in, &now) / 1000;
+        timeout -= inqueue;
+        if (timeout <= 0) {
+            buffer_flush(w->response.data);
+            buffer_strcat(w->response.data, "Query timeout exceeded");
+            return HTTP_RESP_BACKEND_FETCH_FAILED;
+        }
+    }
 
     debug(D_WEB_CLIENT, "%llu: API command 'data' for chart '%s', dimensions '%s', after '%lld', before '%lld', points '%d', group '%d', format '%u', options '0x%08x'"
           , w->id
@@ -588,9 +631,13 @@ inline int web_client_api_request_v1_data(RRDHOST *host, struct web_client *w, c
                 w->id, google_version, google_reqId, google_sig, google_out, responseHandler, outFileName
         );
 
-        buffer_sprintf(w->response.data,
-                "%s({version:'%s',reqId:'%s',status:'ok',sig:'%ld',table:",
-                responseHandler, google_version, google_reqId, st->last_updated.tv_sec);
+        buffer_sprintf(
+            w->response.data,
+            "%s({version:'%s',reqId:'%s',status:'ok',sig:'%"PRId64"',table:",
+            responseHandler,
+            google_version,
+            google_reqId,
+            (int64_t)st->last_updated.tv_sec);
     }
     else if(format == DATASOURCE_JSONP) {
         if(responseHandler == NULL)
@@ -600,10 +647,18 @@ inline int web_client_api_request_v1_data(RRDHOST *host, struct web_client *w, c
         buffer_strcat(w->response.data, "(");
     }
 
-    ret = rrdset2anything_api_v1(st, w->response.data, dimensions, format, points, after, before, group, group_time
-                                 , options, &last_timestamp_in_data, context_param_list, chart_label_key);
+    QUERY_PARAMS query_params = {
+        .context_param_list = context_param_list,
+        .timeout = timeout,
+        .max_anomaly_rates = max_anomaly_rates,
+        .show_dimensions = show_dimensions,
+        .chart_label_key = chart_label_key,
+        .wb = w->response.data};
 
-    free_context_param_list(&context_param_list);
+    ret = rrdset2anything_api_v1(owa, st, &query_params, dimensions, format,
+            points, after, before, group, group_time, options, &last_timestamp_in_data);
+
+    free_context_param_list(owa, &context_param_list);
 
     if(format == DATASOURCE_DATATABLE_JSONP) {
         if(google_timestamp < last_timestamp_in_data)
@@ -620,7 +675,8 @@ inline int web_client_api_request_v1_data(RRDHOST *host, struct web_client *w, c
     else if(format == DATASOURCE_JSONP)
         buffer_strcat(w->response.data, ");");
 
-    cleanup:
+cleanup:
+    onewayalloc_destroy(owa);
     buffer_free(dimensions);
     return ret;
 }
@@ -750,6 +806,7 @@ inline int web_client_api_request_v1_registry(RRDHOST *host, struct web_client *
 
     if(unlikely(action == 'H')) {
         // HELLO request, dashboard ACL
+        analytics_log_dashboard();
         if(unlikely(!web_client_can_access_dashboard(w)))
             return web_client_permission_denied(w);
     }
@@ -866,17 +923,28 @@ static inline void web_client_api_request_v1_info_mirrored_hosts(BUFFER *wb) {
 
         netdata_mutex_lock(&host->receiver_lock);
         buffer_sprintf(
-            wb, "\t\t{ \"guid\": \"%s\", \"reachable\": %s, \"claim_id\": ", host->machine_guid,
-            (host->receiver || host == localhost) ? "true" : "false");
+            wb, "\t\t{ \"guid\": \"%s\", \"hostname\": \"%s\", \"reachable\": %s, \"hops\": %d"
+            , host->machine_guid
+            , host->hostname
+            , (host->receiver || host == localhost) ? "true" : "false"
+            , host->system_info ? host->system_info->hops : (host == localhost) ? 0 : 1
+            );
         netdata_mutex_unlock(&host->receiver_lock);
 
         rrdhost_aclk_state_lock(host);
         if (host->aclk_state.claimed_id)
-            buffer_sprintf(wb, "\"%s\" }", host->aclk_state.claimed_id);
+            buffer_sprintf(wb, ", \"claim_id\": \"%s\"", host->aclk_state.claimed_id);
         else
-            buffer_strcat(wb, "null }");
+            buffer_strcat(wb, ", \"claim_id\": null");
         rrdhost_aclk_state_unlock(host);
 
+        if (host->node_id) {
+            char node_id_str[GUID_LEN + 1];
+            uuid_unparse_lower(*host->node_id, node_id_str);
+            buffer_sprintf(wb, ", \"node_id\": \"%s\" }", node_id_str);
+        } else
+            buffer_strcat(wb, ", \"node_id\": null }");
+
         count++;
     }
     rrd_unlock();
@@ -961,6 +1029,13 @@ inline int web_client_api_request_v1_info_fill_buffer(RRDHOST *host, BUFFER *wb)
     buffer_sprintf(wb, "\t\"container\": \"%s\",\n", (host->system_info->container) ? host->system_info->container : "");
     buffer_sprintf(wb, "\t\"container_detection\": \"%s\",\n", (host->system_info->container_detection) ? host->system_info->container_detection : "");
 
+    if (host->system_info->cloud_provider_type)
+        buffer_sprintf(wb, "\t\"cloud_provider_type\": \"%s\",\n", host->system_info->cloud_provider_type);
+    if (host->system_info->cloud_instance_type)
+        buffer_sprintf(wb, "\t\"cloud_instance_type\": \"%s\",\n", host->system_info->cloud_instance_type);
+    if (host->system_info->cloud_instance_region)
+        buffer_sprintf(wb, "\t\"cloud_instance_region\": \"%s\",\n", host->system_info->cloud_instance_region);
+
     buffer_strcat(wb, "\t\"host_labels\": {\n");
     host_labels2json(host, wb, 2);
     buffer_strcat(wb, "\t},\n");
@@ -978,13 +1053,18 @@ inline int web_client_api_request_v1_info_fill_buffer(RRDHOST *host, BUFFER *wb)
 
 #ifdef ENABLE_ACLK
     buffer_strcat(wb, "\t\"cloud-available\": true,\n");
-#ifdef ACLK_NG
+    buffer_strcat(wb, "\t\"aclk-ng-available\": true,\n");
+#ifdef ENABLE_NEW_CLOUD_PROTOCOL
+    buffer_strcat(wb, "\t\"aclk-ng-new-cloud-protocol\": true,\n");
+#else
+    buffer_strcat(wb, "\t\"aclk-ng-new-cloud-protocol\": false,\n");
+#endif
+    buffer_strcat(wb, "\t\"aclk-legacy-available\": false,\n");
     buffer_strcat(wb, "\t\"aclk-implementation\": \"Next Generation\",\n");
-#else
-    buffer_strcat(wb, "\t\"aclk-implementation\": \"legacy\",\n");
-#endif
 #else
     buffer_strcat(wb, "\t\"cloud-available\": false,\n");
+    buffer_strcat(wb, "\t\"aclk-ng-available\": false,\n");
+    buffer_strcat(wb, "\t\"aclk-legacy-available\": false,\n");
 #endif
     char *agent_id = is_agent_claimed();
     if (agent_id == NULL)
@@ -994,16 +1074,225 @@ inline int web_client_api_request_v1_info_fill_buffer(RRDHOST *host, BUFFER *wb)
         freez(agent_id);
     }
 #ifdef ENABLE_ACLK
-    if (aclk_connected)
-        buffer_strcat(wb, "\t\"aclk-available\": true\n");
+    if (aclk_connected) {
+        buffer_strcat(wb, "\t\"aclk-available\": true,\n");
+#ifdef ENABLE_NEW_CLOUD_PROTOCOL
+        if (aclk_use_new_cloud_arch)
+            buffer_strcat(wb, "\t\"aclk-available-protocol\": \"New\",\n");
+        else
+#endif
+            buffer_strcat(wb, "\t\"aclk-available-protocol\": \"Legacy\",\n");
+    }
     else
 #endif
-        buffer_strcat(wb, "\t\"aclk-available\": false\n");     // Intentionally valid with/without #ifdef above
+        buffer_strcat(wb, "\t\"aclk-available\": false,\n\t\"aclk-available-protocol\": null,\n");     // Intentionally valid with/without #ifdef above
 
-    buffer_strcat(wb, "}");
+    buffer_strcat(wb, "\t\"memory-mode\": ");
+    analytics_get_data(analytics_data.netdata_config_memory_mode, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"multidb-disk-quota\": ");
+    analytics_get_data(analytics_data.netdata_config_multidb_disk_quota, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"page-cache-size\": ");
+    analytics_get_data(analytics_data.netdata_config_page_cache_size, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"stream-enabled\": ");
+    analytics_get_data(analytics_data.netdata_config_stream_enabled, wb);
+    buffer_strcat(wb, ",\n");
+
+#ifdef  ENABLE_COMPRESSION
+    if(host->sender){
+        buffer_strcat(wb, "\t\"stream-compression\": ");
+        buffer_strcat(wb, (host->sender->rrdpush_compression ? "true" : "false"));
+        buffer_strcat(wb, ",\n");
+    }else{
+        buffer_strcat(wb, "\t\"stream-compression\": null,\n");
+    }
+#else
+    buffer_strcat(wb, "\t\"stream-compression\": null,\n");
+#endif  //ENABLE_COMPRESSION   
+
+    buffer_strcat(wb, "\t\"hosts-available\": ");
+    analytics_get_data(analytics_data.netdata_config_hosts_available, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"https-enabled\": ");
+    analytics_get_data(analytics_data.netdata_config_https_enabled, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"buildinfo\": ");
+    analytics_get_data(analytics_data.netdata_buildinfo, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"release-channel\": ");
+    analytics_get_data(analytics_data.netdata_config_release_channel, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"web-enabled\": ");
+    analytics_get_data(analytics_data.netdata_config_web_enabled, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"notification-methods\": ");
+    analytics_get_data(analytics_data.netdata_notification_methods, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"exporting-enabled\": ");
+    analytics_get_data(analytics_data.netdata_config_exporting_enabled, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"exporting-connectors\": ");
+    analytics_get_data(analytics_data.netdata_exporting_connectors, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"allmetrics-prometheus-used\": ");
+    analytics_get_data(analytics_data.netdata_allmetrics_prometheus_used, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"allmetrics-shell-used\": ");
+    analytics_get_data(analytics_data.netdata_allmetrics_shell_used, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"allmetrics-json-used\": ");
+    analytics_get_data(analytics_data.netdata_allmetrics_json_used, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"dashboard-used\": ");
+    analytics_get_data(analytics_data.netdata_dashboard_used, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"charts-count\": ");
+    analytics_get_data(analytics_data.netdata_charts_count, wb);
+    buffer_strcat(wb, ",\n");
+
+    buffer_strcat(wb, "\t\"metrics-count\": ");
+    analytics_get_data(analytics_data.netdata_metrics_count, wb);
+
+#if defined(ENABLE_ML)
+    buffer_strcat(wb, ",\n");
+    char *ml_info = ml_get_host_info(host);
+
+    buffer_strcat(wb, "\t\"ml-info\": ");
+    buffer_strcat(wb, ml_info);
+
+    free(ml_info);
+#endif
+
+    buffer_strcat(wb, "\n}");
     return 0;
 }
 
+#if defined(ENABLE_ML)
+int web_client_api_request_v1_anomaly_events(RRDHOST *host, struct web_client *w, char *url) {
+    if (!netdata_ready)
+        return HTTP_RESP_BACKEND_FETCH_FAILED;
+
+    uint32_t after = 0, before = 0;
+
+    while (url) {
+        char *value = mystrsep(&url, "&");
+        if (!value || !*value)
+            continue;
+
+        char *name = mystrsep(&value, "=");
+        if (!name || !*name)
+            continue;
+        if (!value || !*value)
+            continue;
+
+        if (!strcmp(name, "after"))
+            after = (uint32_t) (strtoul(value, NULL, 0) / 1000);
+        else if (!strcmp(name, "before"))
+            before = (uint32_t) (strtoul(value, NULL, 0) / 1000);
+    }
+
+    char *s;
+    if (!before || !after)
+        s = strdupz("{\"error\": \"missing after/before parameters\" }\n");
+    else {
+        s = ml_get_anomaly_events(host, "AD1", 1, after, before);
+        if (!s)
+            s = strdupz("{\"error\": \"json string is empty\" }\n");
+    }
+
+    BUFFER *wb = w->response.data;
+    buffer_flush(wb);
+
+    wb->contenttype = CT_APPLICATION_JSON;
+    buffer_strcat(wb, s);
+    buffer_no_cacheable(wb);
+
+    freez(s);
+
+    return HTTP_RESP_OK;
+}
+
+int web_client_api_request_v1_anomaly_event_info(RRDHOST *host, struct web_client *w, char *url) {
+    if (!netdata_ready)
+        return HTTP_RESP_BACKEND_FETCH_FAILED;
+
+    uint32_t after = 0, before = 0;
+
+    while (url) {
+        char *value = mystrsep(&url, "&");
+        if (!value || !*value)
+            continue;
+
+        char *name = mystrsep(&value, "=");
+        if (!name || !*name)
+            continue;
+        if (!value || !*value)
+            continue;
+
+        if (!strcmp(name, "after"))
+            after = (uint32_t) strtoul(value, NULL, 0);
+        else if (!strcmp(name, "before"))
+            before = (uint32_t) strtoul(value, NULL, 0);
+    }
+
+    char *s;
+    if (!before || !after)
+        s = strdupz("{\"error\": \"missing after/before parameters\" }\n");
+    else {
+        s = ml_get_anomaly_event_info(host, "AD1", 1, after, before);
+        if (!s)
+            s = strdupz("{\"error\": \"json string is empty\" }\n");
+    }
+
+    BUFFER *wb = w->response.data;
+    buffer_flush(wb);
+    wb->contenttype = CT_APPLICATION_JSON;
+    buffer_strcat(wb, s);
+    buffer_no_cacheable(wb);
+
+    freez(s);
+    return HTTP_RESP_OK;
+}
+
+int web_client_api_request_v1_ml_info(RRDHOST *host, struct web_client *w, char *url) {
+    (void) url;
+
+    if (!netdata_ready)
+        return HTTP_RESP_BACKEND_FETCH_FAILED;
+
+    char *s = ml_get_host_runtime_info(host);
+    if (!s)
+        s = strdupz("{\"error\": \"json string is empty\" }\n");
+
+    BUFFER *wb = w->response.data;
+    buffer_flush(wb);
+    wb->contenttype = CT_APPLICATION_JSON;
+    buffer_strcat(wb, s);
+    buffer_no_cacheable(wb);
+
+    freez(s);
+    return HTTP_RESP_OK;
+}
+
+#endif // defined(ENABLE_ML)
+
 inline int web_client_api_request_v1_info(RRDHOST *host, struct web_client *w, char *url) {
     (void)url;
     if (!netdata_ready) return HTTP_RESP_BACKEND_FETCH_FAILED;
@@ -1017,6 +1306,67 @@ inline int web_client_api_request_v1_info(RRDHOST *host, struct web_client *w, c
     return HTTP_RESP_OK;
 }
 
+static int web_client_api_request_v1_aclk_state(RRDHOST *host, struct web_client *w, char *url) {
+    UNUSED(url);
+    UNUSED(host);
+    if (!netdata_ready) return HTTP_RESP_BACKEND_FETCH_FAILED;
+
+    BUFFER *wb = w->response.data;
+    buffer_flush(wb);
+
+    char *str = aclk_state_json();
+    buffer_strcat(wb, str);
+    freez(str);
+
+    wb->contenttype = CT_APPLICATION_JSON;
+    buffer_no_cacheable(wb);
+    return HTTP_RESP_OK;
+}
+
+int web_client_api_request_v1_metric_correlations(RRDHOST *host, struct web_client *w, char *url) {
+    if (!netdata_ready)
+        return HTTP_RESP_BACKEND_FETCH_FAILED;
+
+    long long baseline_after = 0, baseline_before = 0, highlight_after = 0, highlight_before = 0, max_points = 0;
+ 
+    while (url) {
+        char *value = mystrsep(&url, "&");
+        if (!value || !*value)
+            continue;
+
+        char *name = mystrsep(&value, "=");
+        if (!name || !*name)
+            continue;
+        if (!value || !*value)
+            continue;
+
+        if (!strcmp(name, "baseline_after"))
+            baseline_after = (long long) strtoul(value, NULL, 0);
+        else if (!strcmp(name, "baseline_before"))
+            baseline_before = (long long) strtoul(value, NULL, 0);
+        else if (!strcmp(name, "highlight_after"))
+            highlight_after = (long long) strtoul(value, NULL, 0);
+        else if (!strcmp(name, "highlight_before"))
+            highlight_before = (long long) strtoul(value, NULL, 0);
+        else if (!strcmp(name, "max_points"))
+            max_points = (long long) strtoul(value, NULL, 0);
+        
+    }
+
+    BUFFER *wb = w->response.data;
+    buffer_flush(wb);
+    wb->contenttype = CT_APPLICATION_JSON;
+    buffer_no_cacheable(wb);
+
+    if (!highlight_after || !highlight_before)
+        buffer_strcat(wb, "{\"error\": \"Missing or invalid required highlight after and before parameters.\" }");
+    else {
+        metric_correlations(host, wb, baseline_after, baseline_before, highlight_after, highlight_before, max_points);
+    }
+
+    return HTTP_RESP_OK;
+}
+
 static struct api_command {
     const char *command;
     uint32_t hash;
@@ -1041,7 +1391,16 @@ static struct api_command {
         { "alarm_variables", 0, WEB_CLIENT_ACL_DASHBOARD, web_client_api_request_v1_alarm_variables },
         { "alarm_count",     0, WEB_CLIENT_ACL_DASHBOARD, web_client_api_request_v1_alarm_count     },
         { "allmetrics",      0, WEB_CLIENT_ACL_DASHBOARD, web_client_api_request_v1_allmetrics      },
-        { "manage/health",   0, WEB_CLIENT_ACL_MGMT,      web_client_api_request_v1_mgmt_health     },
+
+#if defined(ENABLE_ML)
+        { "anomaly_events",     0, WEB_CLIENT_ACL_DASHBOARD, web_client_api_request_v1_anomaly_events     },
+        { "anomaly_event_info", 0, WEB_CLIENT_ACL_DASHBOARD, web_client_api_request_v1_anomaly_event_info },
+        { "ml_info",            0, WEB_CLIENT_ACL_DASHBOARD, web_client_api_request_v1_ml_info            },
+#endif
+
+        { "manage/health",       0, WEB_CLIENT_ACL_MGMT,      web_client_api_request_v1_mgmt_health         },
+        { "aclk",                0, WEB_CLIENT_ACL_DASHBOARD, web_client_api_request_v1_aclk_state          },
+        { "metric_correlations", 0, WEB_CLIENT_ACL_DASHBOARD, web_client_api_request_v1_metric_correlations },
         // terminator
         { NULL,              0, WEB_CLIENT_ACL_NONE,      NULL                                      },
 };
